@@ -1,6 +1,8 @@
 use std::collections::BTreeMap;
 use std::rc::Rc;
 
+use log::warn;
+
 use crate::brokers::Broker;
 use crate::broker_statement::{BrokerStatement, ReadingStrictness, StockSellType};
 use crate::commissions::CommissionCalc;
@@ -8,7 +10,9 @@ use crate::config::{Config, PortfolioConfig, PerformanceMergingConfig};
 use crate::core::{GenericResult, EmptyResult};
 use crate::currency::converter::{CurrencyConverter, CurrencyConverterRc};
 use crate::db;
+use crate::localities::Country;
 use crate::quotes::Quotes;
+use crate::taxes::{IncomeType, LtoDeductionCalculator, LtoDeduction, NetLtoDeduction};
 use crate::telemetry::TelemetryRecordBuilder;
 use crate::types::Decimal;
 
@@ -22,12 +26,20 @@ mod portfolio_performance;
 mod sell_simulation;
 
 pub struct PortfolioStatistics {
+    country: Country,
     pub currencies: Vec<PortfolioCurrencyStatistics>,
+    pub lto: Option<LtoStatistics>,
+}
+
+pub struct LtoStatistics {
+    pub applied: BTreeMap<i32, NetLtoDeduction>,
+    pub projected: LtoDeduction,
 }
 
 impl PortfolioStatistics {
-    fn new() -> PortfolioStatistics {
+    fn new(country: Country) -> PortfolioStatistics {
         PortfolioStatistics {
+            country,
             currencies: ["USD", "RUB"].iter().map(|&currency| (
                 PortfolioCurrencyStatistics {
                     currency: currency.to_owned(),
@@ -41,14 +53,33 @@ impl PortfolioStatistics {
                     projected_commissions: dec!(0),
                 }
             )).collect(),
+            lto: None,
         }
     }
 
     pub fn print(&self) {
+        let lto = self.lto.as_ref().unwrap();
+
+        for (year, result) in &lto.applied {
+            if !result.loss.is_zero() {
+                warn!("Long-term ownership tax deduction loss in {}: {}.",
+                      year, self.country.cash(result.loss));
+            }
+
+            if !result.applied_above_limit.is_zero() {
+                warn!("Long-term ownership tax deductions applied in {} have exceeded the total limit by {}.",
+                      year, self.country.cash(result.applied_above_limit));
+            }
+        }
+
         for statistics in &self.currencies {
             statistics.performance.as_ref().unwrap().print(&format!(
                 "Average rate of return from cash investments in {}", &statistics.currency));
         }
+
+        if !lto.projected.deduction.is_zero() {
+            lto.projected.print("Projected LTO deduction")
+        }
     }
 
     fn process<F>(&mut self, mut handler: F) -> EmptyResult
@@ -91,7 +122,8 @@ pub fn analyse(
 
     let country = config.get_tax_country();
     let (converter, quotes) = load_tools(config)?;
-    let mut statistics = PortfolioStatistics::new();
+    let mut lto_calc = LtoDeductionCalculator::new();
+    let mut statistics = PortfolioStatistics::new(country.clone());
 
     for (_, statement) in &mut portfolios {
         statement.batch_quotes(&quotes)?;
@@ -141,15 +173,35 @@ pub fn analyse(
                 StockSellType::Trade {volume, commission, ..} => (volume, commission),
                 _ => unreachable!(),
             };
+
             let (tax_year, _) = portfolio.tax_payment_day().get(trade.execution_date, true);
             let details = trade.calculate(&country, tax_year, &portfolio.tax_exemptions, &converter)?;
 
+            let mut taxable_local_profit = details.taxable_local_profit;
+            for source in &details.fifo {
+                if let Some(deductible) = source.long_term_ownership_deductible {
+                    lto_calc.add(deductible.profit, deductible.years, false);
+                    taxable_local_profit.amount -= deductible.profit;
+                }
+            }
+
+            let tax_without_deduction = country.tax_to_pay(
+                IncomeType::Trading, tax_year, details.local_profit, None);
+
+            let tax_to_pay = country.tax_to_pay(
+                IncomeType::Trading, tax_year, taxable_local_profit, None);
+
+            let tax_deduction = tax_without_deduction - tax_to_pay;
+            assert!(!tax_deduction.is_negative());
+
             statistics.process(|statistics| {
                 let currency = &statistics.currency;
+
                 let volume = converter.real_time_convert_to(volume, currency)?;
                 let commission = converter.real_time_convert_to(commission, currency)?;
-                let tax_to_pay = converter.real_time_convert_to(details.tax_to_pay, currency)?;
-                let tax_deduction = converter.real_time_convert_to(details.tax_deduction, currency)?;
+
+                let tax_to_pay = converter.real_time_convert_to(tax_to_pay, currency)?;
+                let tax_deduction = converter.real_time_convert_to(tax_deduction, currency)?;
 
                 statistics.add_assets(broker, &trade.symbol, volume);
                 statistics.projected_commissions += commission;
@@ -172,6 +224,8 @@ pub fn analyse(
         }
     }
 
+    let mut applied_lto = None;
+
     statistics.process(|statistics| {
         let mut analyser = PortfolioPerformanceAnalyser::new(
             &country, &statistics.currency, &converter, include_closed_positions);
@@ -180,10 +234,22 @@ pub fn analyse(
             analyser.add(&portfolio, &statement)?;
         }
 
-        statistics.performance.replace(analyser.analyse()?);
+        let (performance, lto) = analyser.analyse()?;
+        statistics.performance.replace(performance);
+
+        if let Some(prev) = applied_lto.take() {
+            assert_eq!(prev, lto);
+        }
+        applied_lto.replace(lto);
+
         Ok(())
     })?;
 
+    statistics.lto = Some(LtoStatistics {
+        applied: applied_lto.unwrap(),
+        projected: lto_calc.calculate()
+    });
+
     Ok((statistics, converter, telemetry))
 }
 
