@@ -9,7 +9,7 @@ use crate::currency::Cash;
 use crate::currency::converter::CurrencyConverter;
 use crate::formatting;
 use crate::localities::Country;
-use crate::taxes::NetTaxCalculator;
+use crate::taxes::{NetTax, NetTaxCalculator, NetLtoDeduction, NetLtoDeductionCalculator};
 use crate::time::{self, Date, DateOptTime};
 use crate::types::Decimal;
 
@@ -30,6 +30,7 @@ pub struct PortfolioPerformanceAnalyser<'a> {
     transactions: Vec<Transaction>,
     income_structure: IncomeStructure,
     instruments: Option<BTreeMap<String, StockDepositView>>,
+    net_lto_calc: NetLtoDeductionCalculator,
     current_assets: Decimal,
 }
 
@@ -48,6 +49,7 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
             transactions: Vec::new(),
             income_structure: Default::default(),
             instruments: Some(BTreeMap::new()),
+            net_lto_calc: NetLtoDeductionCalculator::new(),
             current_assets: dec!(0),
         }
     }
@@ -77,7 +79,7 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
         Ok(())
     }
 
-    pub fn analyse(mut self) -> GenericResult<PortfolioPerformanceAnalysis> {
+    pub fn analyse(mut self) -> GenericResult<(PortfolioPerformanceAnalysis, BTreeMap<i32, NetLtoDeduction>)> {
         let mut instrument_performance = BTreeMap::new();
 
         self.calculate_open_position_periods()?;
@@ -94,11 +96,11 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
         let portfolio_performance = self.analyse_portfolio_performance()?;
         self.income_structure.net_profit = portfolio_performance.net_profit();
 
-        Ok(PortfolioPerformanceAnalysis {
+        Ok((PortfolioPerformanceAnalysis {
             income_structure: self.income_structure,
             instruments: instrument_performance,
             portfolio: portfolio_performance,
-        })
+        }, self.net_lto_calc.calculate()))
     }
 
     fn analyse_instrument_performance(
@@ -317,17 +319,27 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
                     }
 
                     let (tax_year, _) = portfolio.tax_payment_day().get(trade.execution_date, true);
-                    let details = trade.calculate(
-                        &self.country, tax_year, &portfolio.tax_exemptions, self.converter)?;
+                    let details = trade.calculate(&self.country, tax_year, &portfolio.tax_exemptions, self.converter)?;
 
-                    let local_profit = details.local_profit.amount;
-                    let taxable_local_profit = details.taxable_local_profit.amount;
+                    let mut lto_deductibles = Vec::new();
+
+                    for fifo in &details.fifo {
+                        if let Some(lto) = fifo.long_term_ownership_deductible {
+                            self.net_lto_calc.add_profit(tax_year, lto.profit, lto.years, trade.emulation);
+                            lto_deductibles.push(lto);
+                        }
+                    }
 
                     stock_taxes.entry(&trade.symbol)
-                        .or_insert_with(|| NetTaxCalculator::new(self.country.clone(), portfolio.tax_payment_day()))
-                        .add_profit(trade.execution_date, local_profit, taxable_local_profit);
+                        .or_insert_with(|| NetTaxCalculator::new(
+                            self.country.clone(), portfolio.tax_payment_day()))
+                        .add_profit(
+                            trade.execution_date, details.local_profit, details.taxable_local_profit,
+                            &lto_deductibles, trade.emulation);
 
-                    taxes.add_profit(trade.execution_date, local_profit, taxable_local_profit);
+                    taxes.add_profit(
+                        trade.execution_date, details.local_profit, details.taxable_local_profit,
+                        &lto_deductibles, trade.emulation);
                 },
                 StockSellType::CorporateAction => {
                     let deposit_view = self.get_deposit_view(&trade.symbol);
@@ -336,18 +348,21 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
             };
         }
 
-        for (&symbol, symbol_taxes) in stock_taxes.iter() {
-            for (&tax_payment_date, &(tax_to_pay, _tax_deduction)) in symbol_taxes.get_taxes().iter() {
+        for (symbol, symbol_taxes) in stock_taxes.into_iter() {
+            for (_, NetTax{tax_payment_date, tax_to_pay, ..}) in symbol_taxes.calculate().into_iter() {
                 if let Some(amount) = self.map_tax_to_deposit_amount(tax_payment_date, tax_to_pay)? {
                     trace!("* {} selling {} tax: {}",
                            symbol, formatting::format_date(tax_payment_date), amount);
 
-                    self.get_deposit_view(symbol).transaction(tax_payment_date.into(), amount);
+                    self.get_deposit_view(&symbol).transaction(tax_payment_date.into(), amount);
                 }
             }
         }
 
-        for (&tax_payment_date, &(tax_to_pay, tax_deduction)) in taxes.get_taxes().iter() {
+        for (tax_year, NetTax {
+            tax_payment_date, tax_to_pay, tax_deduction,
+            lto_deduction, lto_loss,
+        }) in taxes.calculate().into_iter() {
             if let Some(amount) = self.map_tax_to_deposit_amount(tax_payment_date, tax_to_pay)? {
                 trace!("* Stock selling {} tax: {}", formatting::format_date(tax_payment_date), amount);
                 self.transaction(tax_payment_date, amount);
@@ -358,6 +373,8 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
                 trace!("* {} tax deduction: {}", formatting::format_date(tax_payment_date), amount);
                 self.income_structure.tax_deductions += amount;
             }
+
+            self.net_lto_calc.add_applied_deduction(tax_year, lto_deduction.amount, lto_loss.amount);
         }
 
         Ok(())
@@ -448,16 +465,14 @@ impl <'a> PortfolioPerformanceAnalyser<'a> {
         self.transactions.push(Transaction::new(date, amount));
     }
 
-    fn map_tax_to_deposit_amount(&self, tax_payment_date: Date, tax_to_pay: Decimal) -> GenericResult<Option<Decimal>> {
+    fn map_tax_to_deposit_amount(&self, tax_payment_date: Date, tax_to_pay: Cash) -> GenericResult<Option<Decimal>> {
         // Treat tax payment as an ordinary deposit which we transfer to the account at tax payment
         // day.
 
         if tax_to_pay.is_zero() {
             return Ok(None);
         }
-        assert!(tax_to_pay.is_sign_positive());
-
-        let tax_to_pay = Cash::new(self.country.currency, tax_to_pay);
+        assert!(tax_to_pay.is_positive());
 
         let conversion_date = if tax_payment_date > self.today {
             self.today
