@@ -5,7 +5,7 @@ use lazy_static::lazy_static;
 use num_traits::ToPrimitive;
 use prometheus::{self, TextEncoder, Encoder, Gauge, GaugeVec, register_gauge, register_gauge_vec};
 
-use crate::analysis::{self, PortfolioCurrencyStatistics};
+use crate::analysis::{self, PortfolioCurrencyStatistics, LtoStatistics};
 use crate::config::Config;
 use crate::core::{EmptyResult, GenericError, GenericResult};
 use crate::currency::converter::CurrencyConverter;
@@ -15,40 +15,46 @@ use crate::types::Decimal;
 
 lazy_static! {
     static ref UPDATE_TIME: Gauge = register_simple_metric(
-        "time", "Metrics generation time.");
+        "time", "Metrics generation time");
 
     static ref BROKERS: GaugeVec = register_metric(
-        "brokers", "Net asset value by broker.", &["currency", "broker", "country"]);
+        "brokers", "Net asset value by broker", &["currency", "broker", "country"]);
 
     static ref ASSETS: GaugeVec = register_instrument_metric(
-        "assets", "Open positions value.");
+        "assets", "Open positions value");
 
     static ref PERFORMANCE: GaugeVec = register_instrument_metric(
-        "performance", "Instrument performance.");
+        "performance", "Instrument performance");
 
     static ref INCOME_STRUCTURE: GaugeVec = register_structure_metric(
-        "income_structure", "Income structure.");
+        "income_structure", "Income structure");
 
     static ref EXPENCES_STRUCTURE: GaugeVec = register_structure_metric(
-        "expenses_structure", "Expenses structure.");
+        "expenses_structure", "Expenses structure");
 
     static ref PROFIT: GaugeVec = register_portfolio_metric(
-        "profit", "Profit.");
+        "profit", "Profit");
 
     static ref NET_PROFIT: GaugeVec = register_portfolio_metric(
-        "net_profit", "Net profit.");
+        "net_profit", "Net profit");
 
     static ref PROJECTED_TAXES: GaugeVec = register_portfolio_metric(
-        "projected_taxes", "Projected taxes to pay.");
+        "projected_taxes", "Projected taxes to pay");
 
     static ref PROJECTED_TAX_DEDUCTIONS: GaugeVec = register_portfolio_metric(
-        "projected_tax_deductions", "Projected tax deductions.");
+        "projected_tax_deductions", "Projected tax deductions");
 
     static ref PROJECTED_COMMISSIONS: GaugeVec = register_portfolio_metric(
-        "projected_commissions", "Projected commissions to pay.");
+        "projected_commissions", "Projected commissions to pay");
+
+    static ref LTO: GaugeVec = register_metric(
+        "lto", "Long-term ownership tax exemption applying results", &["year", "type"]);
+
+    static ref PROJECTED_LTO: GaugeVec = register_metric(
+        "projected_lto", "Long-term ownership tax exemption projected results", &["type"]);
 
     static ref FOREX_PAIRS: GaugeVec = register_metric(
-        "forex_pairs", "Forex quotes.", &["base", "quote"]);
+        "forex_pairs", "Forex quotes", &["base", "quote"]);
 }
 
 pub fn collect(config: &Config, path: &str) -> GenericResult<TelemetryRecordBuilder> {
@@ -57,11 +63,13 @@ pub fn collect(config: &Config, path: &str) -> GenericResult<TelemetryRecordBuil
 
     UPDATE_TIME.set(cast::f64(time::utc_now().timestamp()));
 
-    for statistics in statistics.currencies {
-        collect_portfolio_metrics(&statistics);
+    for statistics in &statistics.currencies {
+        collect_portfolio_metrics(statistics);
     }
 
+    collect_lto_metrics(statistics.lto.as_ref().unwrap());
     collect_forex_quotes(&converter, "USD", "RUB")?;
+
     save(path)?;
 
     Ok(telemetry)
@@ -106,6 +114,20 @@ fn collect_portfolio_metrics(statistics: &PortfolioCurrencyStatistics) {
     set_portfolio_metric(&PROJECTED_COMMISSIONS, currency, statistics.projected_commissions);
 }
 
+fn collect_lto_metrics(lto: &LtoStatistics) {
+    for (year, result) in &lto.applied {
+        let year = year.to_string();
+        let year = year.as_str();
+
+        set_metric(&LTO, &[year, "applied-above-limit"], result.applied_above_limit);
+        set_metric(&LTO, &[year, "loss"], result.loss);
+    }
+
+    set_metric(&PROJECTED_LTO, &["deduction"], lto.projected.deduction);
+    set_metric(&PROJECTED_LTO, &["limit"], lto.projected.limit);
+    set_metric(&PROJECTED_LTO, &["loss"], lto.projected.loss);
+}
+
 fn collect_forex_quotes(converter: &CurrencyConverter, base: &str, quote: &str) -> EmptyResult {
     Ok(set_metric(&FOREX_PAIRS, &[base, quote], converter.real_time_currency_rate(base, quote)?))
 }
